#!/usr/bin/perl
#
# Copyright (C) 2023 Juan Domingo (Juan.Domingo@uv.es)
#
# This script is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# This is the second version of the macro to fill the Pathology table with the ICD10 codes.
# Now we use the ICD10-2021, as stated in file icd10cm_order_2021.txt. Format is different from the old one.

use strict;
use warnings;

($#ARGV == 0) || die("Usage: $0 <ICD_file.txt>\n");

# The format of the file is a join of five columns like
# 00001 A00     0 Cholera                                                      Cholera
# There are no tabs. The number of spaces is meant to make each column start aligned. 

# Columns are:
#  Consecutive number with five digits (preceeded by 0 if needed)
#  ICD code (up to 6 characters, the first an upper-case letter, followed by some numbers and possibly digits at the end)
#  A single digit, either 0 or 1 (what is this, anyway?)
#  Abbreviated pathology name (so as to fit in up to 60 characters)
#  Full pathology name

# We are interested in columns 2 and 5 (code and fullname). They span respectively through positions (numbered
# from 0) 6-13 (both included) and 77-end_of_line. These blocks will be isolated and their spaces
# at the end of the chunck will be elliminated. Only codes not ending in letters will be introduced, since the
# letters at the end are variations of the same pathology and we are not currently interested in such level of
# details (at lest, not now. It might change in the future but that would imply additions to the DB, not deletions).

open(F,"<$ARGV[0]") || die("Cannot open input file.\n");
open(G,">FillPathology.sql");

my $line;
my $code;
my $name;
my $numpat=0;
my $numlines=0;
my $maxlen=0;
while (defined($line=<F>))
{
 chomp($line);
 $code=substr($line,6,7);
 $code =~ s/[\s\t]+$//g;
 if (!($code =~ /[A-Z]$/))
 {
  if (length($code)>3)
  {
   if ($code =~ /([A-Z])(\d)([A-Z\d])(.*)/)
   {
    $code = $1.$2.$3.".".$4;
   }
   else
   {
    die("Error: code $code is longer than three characters but does not follow the schema Letter/Digit/Digit/(more digits...)\n");
   }
  }
  $name=substr($line,77); # No length means 'up to the end'
  $name =~ s/[\s\t]+$//g;
  # Now, substitute the ticks into the stringby \'
  $name =~ s/\'/\\\'/g;
  print G "INSERT INTO Pathology VALUES('".$code."',";
  print G ' ' x (8-length($code));
  print G "'".$name."');\n";
  if (length($name)>$maxlen)
  {
   $maxlen=length($name);
  }
  $numpat++;
 } 
 $numlines++;
}
close(G);
close(F);
print "$numpat pathologies selected out of $numlines.\nMaximal length of name is $maxlen characters.\n";

